--[[============================================================================
main.lua
============================================================================]]--

-- Reload the script whenever this file is saved. 
-- Additionally, execute the attached function.
_AUTO_RELOAD_DEBUG = function()
  
end

-- Read from the manifest.xml file.
class "RenoiseScriptingTool" (renoise.Document.DocumentNode)
  function RenoiseScriptingTool:__init()    
    renoise.Document.DocumentNode.__init(self) 
    self:add_property("Name", "Untitled Tool")
    self:add_property("Id", "Unknown Id")
  end

local manifest = RenoiseScriptingTool()
local ok,err = manifest:load_from("manifest.xml")
local tool_name = manifest:property("Name").value
local tool_id = manifest:property("Id").value

local asslm_active = false

--------------------------------------------------------------------------------
-- Main functions
--------------------------------------------------------------------------------

-- This example function is called from the GUI below.
-- It will return a random string. The GUI function displays 
-- that string in a dialog.

local function build_notifier(obs, ntfr)
  if not obs:has_notifier(ntfr) then
    obs:add_notifier(ntfr)
    return true
  else
    return false
  end
end

local function strip_notifier(obs, ntfr)
  if obs:has_notifier(ntfr) then
    obs:remove_notifier(ntfr)
    return true
  else
    return false
  end
end

local function update_pattern_selection()
--  {
--    start_line,     -- Start pattern line index
--    start_track,    -- Start track index
--    start_column,   -- Start column index within start_track   
-- 
--    end_line,       -- End pattern line index
--    end_track,      -- End track index
--    end_column      -- End column index within end_track
--  }
  local epos = renoise.song().transport.edit_pos
  local line = epos.line
  local track = renoise.song().selected_track_index
  local column
  if renoise.song().selected_note_column then
    column = renoise.song().selected_note_column_index
  else
    column = renoise.song().selected_track.visible_note_columns + renoise.song().selected_effect_column_index
  end
  show_msg(string.format("AUTO SELECTED LINE:%x, TRACK:%d, COLUMN:%d",line ,track ,column))
  renoise.song().selection_in_pattern = {
    --start
    start_line = line,
    start_track = track,
    start_column = column,
    --end
    end_line = line,
    end_track = track,
    end_column = column
  }
end

local function toggle_mode()

  asslm_active = not asslm_active

  if asslm_active == true then
    --info
    show_msg("AUTO SELECT SINGLE LINE MODE ACTIVE")
    --setup notifiers to constantly update selection on edit position
    build_notifier(renoise.tool().app_idle_observable, update_pattern_selection)
  else
    --info
    show_msg("auto select single line mode off")
    --remove notifiers to constantly update selection on edit position
    strip_notifier(renoise.tool().app_idle_observable, update_pattern_selection)
  end
  
end


--------------------------------------------------------------------------------
-- GUI
--------------------------------------------------------------------------------

function show_msg(msg)
  renoise.app():show_status(msg)
end


--------------------------------------------------------------------------------
-- Menu entries
--------------------------------------------------------------------------------

--[[
renoise.tool():add_menu_entry {
  name = "Main Menu:Tools:"..tool_name.."...",
  invoke = show_dialog  
}
--]]

--------------------------------------------------------------------------------
-- Key Binding
--------------------------------------------------------------------------------

renoise.tool():add_keybinding {
  name = "Pattern Editor:Navigation:Toggle AutoSingleLineSelect.",
  invoke = toggle_mode
}


--------------------------------------------------------------------------------
-- MIDI Mapping
--------------------------------------------------------------------------------

--[[
renoise.tool():add_midi_mapping {
  name = tool_id..":Show Dialog...",
  invoke = show_dialog
}
--]]
